﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Media;

namespace AudioStreamAgent
{
    public class Sine440AudioStreamSource : MediaStreamSource
    {
        // Standard constants
        const int ChannelCount = 1;
        const int BitsPerSample = 16;
        const int BufferSamples = 4096;     // can be changed
        const int BufferSize = ChannelCount * BufferSamples * BitsPerSample / 8;

        Dictionary<MediaSampleAttributeKeys, string> mediaSampleAttributes;
        double angle;
        double angleIncrement;
        int sampleRate;
        long timestamp;
        MediaStreamDescription mediaStreamDescription;
        MemoryStream memoryStream;

        public Sine440AudioStreamSource(int sampleRate)
        {
            this.sampleRate = sampleRate;

            // Want to play A = 440Hz
            angleIncrement = 2 * Math.PI * 440 / sampleRate;

            // Create empty mediaSampleAttributes dictionary for OpenReadAsync
            mediaSampleAttributes = new Dictionary<MediaSampleAttributeKeys, string>();

            // Create re-usable MemoryStream for accumulating audio samples
            memoryStream = new MemoryStream();
        }

        protected override void OpenMediaAsync()
        {
            int byteRate = sampleRate * ChannelCount * BitsPerSample / 8;
            short blockAlign = (short)(ChannelCount * (BitsPerSample / 8));

            // Build string-based wave-format structure
            string waveFormat = "";
            waveFormat += ToLittleEndianString(string.Format("{0:X4}", 1));      // indicates PCM
            waveFormat += ToLittleEndianString(string.Format("{0:X4}", ChannelCount));
            waveFormat += ToLittleEndianString(string.Format("{0:X8}", sampleRate));
            waveFormat += ToLittleEndianString(string.Format("{0:X8}", byteRate));
            waveFormat += ToLittleEndianString(string.Format("{0:X4}", blockAlign));
            waveFormat += ToLittleEndianString(string.Format("{0:X4}", BitsPerSample));
            waveFormat += ToLittleEndianString(string.Format("{0:X4}", 0));

            // Put wave format string in media streams dictionary
            var mediaStreamAttributes = new Dictionary<MediaStreamAttributeKeys, string>();
            mediaStreamAttributes[MediaStreamAttributeKeys.CodecPrivateData] = waveFormat;

            // Make description to add to available streams list
            var availableMediaStreams = new List<MediaStreamDescription>();
            mediaStreamDescription = new MediaStreamDescription(MediaStreamType.Audio, mediaStreamAttributes);
            availableMediaStreams.Add(mediaStreamDescription);

            // Set some appropriate keys in the media source dictionary
            var mediaSourceAttributes = new Dictionary<MediaSourceAttributesKeys, string>();
            mediaSourceAttributes[MediaSourceAttributesKeys.Duration] = "0";
            mediaSourceAttributes[MediaSourceAttributesKeys.CanSeek] = "false";

            // Signal that the open operation is completed
            ReportOpenMediaCompleted(mediaSourceAttributes, availableMediaStreams);
        }

        // For building string-based wave-format structure
        string ToLittleEndianString(string bigEndianString)
        {
            StringBuilder strBuilder = new StringBuilder();

            for (int i = 0; i < bigEndianString.Length; i += 2)
                strBuilder.Insert(0, bigEndianString.Substring(i, 2));

            return strBuilder.ToString();
        }

        // Provides audio samples from AudioSampleProvider property.
        //  (MediaStreamType parameter will always equal Audio.)
        protected override void GetSampleAsync(MediaStreamType mediaStreamType)
        {
            // Reset MemoryStream object
            memoryStream.Seek(0, SeekOrigin.Begin);

            for (int sample = 0; sample < BufferSamples; sample++)
            {
                short amplitude = (short)(short.MaxValue * Math.Sin(angle));  
                memoryStream.WriteByte((byte)(amplitude & 0xFF));
                memoryStream.WriteByte((byte)(amplitude >> 8));
                angle = (angle + angleIncrement) % (2 * Math.PI);
            }

            // Send out the sample
            ReportGetSampleCompleted(new MediaStreamSample(mediaStreamDescription,
                                                           memoryStream,
                                                           0,
                                                           BufferSize,
                                                           timestamp,
                                                           mediaSampleAttributes));
            // Prepare for next sample
            timestamp += BufferSamples * 10000000L / sampleRate;
        }

        protected override void SeekAsync(long seekToTime)
        {
            ReportSeekCompleted(seekToTime);
        }

        protected override void CloseMedia()
        {
            mediaStreamDescription = null;
        }

        // Shouldn't get a call here because only one MediaStreamDescription is supported
        protected override void SwitchMediaStreamAsync(MediaStreamDescription mediaStreamDescription)
        {
            throw new NotImplementedException();
        }

        protected override void GetDiagnosticAsync(MediaStreamSourceDiagnosticKind diagnosticKind)
        {
            throw new NotImplementedException();
        }
    }
}
