﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace Petzold.MusicSynthesis.Controls
{
    public partial class Waveformer : UserControl
    {
        const int NUM_POINTS = 15;
        Dictionary<int, WaveformerNode> touchDictionary = new Dictionary<int, WaveformerNode>();

        public Waveformer()
        {
            InitializeComponent();

            this.Points = new FixedLengthCollection<Point>(NUM_POINTS);
            this.Points.CollectionChanged += new NotifyCollectionChangedEventHandler(OnPointsCollectionChanged);

            for (int i = 0; i < NUM_POINTS; i++)
            {
                canvas.Children.Add(new WaveformerNode());
                polyline.Points.Add(new Point());
            }

            // Add two more to Polyline for beginning and end
            polyline.Points.Add(new Point());
            polyline.Points.Add(new Point());

            SizeChanged += OnSizeChanged;
            Touch.FrameReported += OnTouchFrameReported;
        }

        public FixedLengthCollection<Point> Points { protected set; get; }

        void OnSizeChanged(object sender, SizeChangedEventArgs args)
        {
            Synchronize();
        }

        void OnPointsCollectionChanged(object sender, NotifyCollectionChangedEventArgs args)
        {
            Synchronize();
        }

        void Synchronize()
        {
            // First and last points in Polyline are on axis
            polyline.Points[0] = RelativeToAbsolute(new Point(0, 0)); 
            polyline.Points[NUM_POINTS + 1] = RelativeToAbsolute(new Point(2 * Math.PI, 0));

            for (int index = 0; index < NUM_POINTS; index++)
            {
                Point point = RelativeToAbsolute(this.Points[index]);
                (canvas.Children[index] as WaveformerNode).Point = point;
                polyline.Points[index + 1] = point;
            }
        }

        void OnTouchFrameReported(object sender, TouchFrameEventArgs args)
        {
            TouchPoint primaryTouchPoint = args.GetPrimaryTouchPoint(waveLayoutPanel);

            // Don't let usable touch points get promoted to mouse events
            if (primaryTouchPoint != null &&
                    primaryTouchPoint.Action == TouchAction.Down &&
                    GetParent<WaveformerNode>(primaryTouchPoint.TouchDevice.DirectlyOver) is WaveformerNode)
                args.SuspendMousePromotionUntilTouchUp();

            // Enumerate all the TouchPoints
            TouchPointCollection touchPoints = args.GetTouchPoints(waveLayoutPanel);

            foreach (TouchPoint touchPoint in touchPoints)
            {
                TouchDevice touchDevice = touchPoint.TouchDevice;
                int id = touchDevice.Id;
                Point position = touchPoint.Position;

                switch (touchPoint.Action)
                {
                    case TouchAction.Down:
                        WaveformerNode node = GetParent<WaveformerNode>(touchDevice.DirectlyOver);

                        if (node != null)
                        {
                            // Shouldn't happen but let it go
                            if (touchDictionary.ContainsKey(id))
                                touchDictionary[id] = node;
                            else
                                // Normal case
                                touchDictionary.Add(id, node);

                            SetPointForNode(node, position);                            
                        }
                        break;

                    case TouchAction.Move:
                        if (touchDictionary.ContainsKey(id))
                            SetPointForNode(touchDictionary[id], position);
                        break;

                    case TouchAction.Up:
                        if (touchDictionary.ContainsKey(id))
                            touchDictionary.Remove(id);
                        break;
                }
            }
        }

        T GetParent<T>(DependencyObject element) where T : DependencyObject
        {
            DependencyObject parent = VisualTreeHelper.GetParent(element);

            if (parent == null)
                return null;

            if (parent is T)
                return parent as T;

            return GetParent<T>(parent);
        }

        void SetPointForNode(WaveformerNode node, Point position)
        {
            int childIndex = canvas.Children.IndexOf(node);
            double x = this.Points[childIndex].X;
            double y = AbsoluteToRelativeY(position.Y);
            this.Points[childIndex] = new Point(x, y);
        }

        Point RelativeToAbsolute(Point point)
        {
            return new Point(waveLayoutPanel.ActualWidth * point.X / (2 * Math.PI),
                             waveLayoutPanel.ActualHeight * (1 - point.Y) / 2);
        }

        double AbsoluteToRelativeY(double y)
        {
            return Math.Max(-1, Math.Min(1, 1 - 2 * y / waveLayoutPanel.ActualHeight));
        }
    }
}
