﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows;
using System.Windows.Media;
using System.Windows.Navigation;
using System.Xml.Serialization;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Petzold.MusicSynthesis;

namespace WaveformManipulator
{
    public partial class MainPage : PhoneApplicationPage
    {
        const int SAMPLE_RATE = 22050;
        Mixer mixer;

        public MainPage()
        {
            InitializeComponent();

            // Get app bar buttons
            playButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            pauseButton = this.ApplicationBar.Buttons[1] as ApplicationBarIconButton;

            // Initialize Waveformer control
            for (int i = 0; i < waveformer.Points.Count; i++)
            {
                double angle = (i + 1) * 2 * Math.PI / (waveformer.Points.Count + 1);
                waveformer.Points[i] = new Point(angle, Math.Sin(angle));
            }

            // Create two Sequencers with slightly different tempi
            Sequencer sequencer1 = new Sequencer(SAMPLE_RATE)
            {
                Oscillator = new VariableWaveformOscillator(SAMPLE_RATE)
                {
                    Points = waveformer.Points
                },
                Tempo = 480
            };

            Sequencer sequencer2 = new Sequencer(SAMPLE_RATE)
            {
                Oscillator = new VariableWaveformOscillator(SAMPLE_RATE)
                {
                    Points = waveformer.Points
                },
                Tempo = 470
            };

            // Set the same Pitches in the Sequencers
            Pitch[] pitches = 
            {
                new Pitch(Note.E, 4),
                new Pitch(Note.G, 4),
                new Pitch(Note.B, 4),
                new Pitch(Note.E, 5),
                new Pitch(Note.B, 4),
                new Pitch(Note.G, 4),

                new Pitch(Note.A, 4),
                new Pitch(Note.C, 5),
                new Pitch(Note.E, 5),
                new Pitch(Note.A, 5),
                new Pitch(Note.E, 5),
                new Pitch(Note.C, 5),

                new Pitch(Note.D, 4),
                new Pitch(Note.F, 4),
                new Pitch(Note.A, 4),
                new Pitch(Note.D, 5),
                new Pitch(Note.A, 4),
                new Pitch(Note.F, 4),

                new Pitch(Note.G, 4),
                new Pitch(Note.B, 4),
                new Pitch(Note.D, 5),
                new Pitch(Note.G, 5),
                new Pitch(Note.D, 5),
                new Pitch(Note.B, 4)
            };

            foreach (Pitch pitch in pitches)
            {
                sequencer1.Pitches.Add(pitch);
                sequencer2.Pitches.Add(pitch);
            }

            // Create Mixer and MixerInput objects
            mixer = new Mixer();
            mixer.Inputs.Add(new MixerInput(sequencer1) { Space = -0.5 });
            mixer.Inputs.Add(new MixerInput(sequencer2) { Space = 0.5 });
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            Point[] points = new Point[waveformer.Points.Count];

            using (IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (storage.FileExists("settings.xml"))
                {
                    // Load Waveformer Points
                    XmlSerializer xmlSerializer = new XmlSerializer(typeof(Point[]));
                    using (IsolatedStorageFileStream stream = storage.OpenFile("settings.xml", FileMode.Open))
                    {
                        points = xmlSerializer.Deserialize(stream) as Point[];
                    }
                }
                else
                {
                    // Or initialize them to sine curve
                    for (int i = 0; i < waveformer.Points.Count; i++)
                    {
                        double angle = (i + 1) * 2 * Math.PI / (waveformer.Points.Count + 1);
                        points[i] = new Point(angle, Math.Sin(angle));
                    }
                }
            }

            for (int i = 0; i < waveformer.Points.Count; i++)
                waveformer.Points[i] = points[i];

            // Create MediaStreamSource derivative & set in MediaElement
            DynamicPcmStreamSource dynamicPcmStreamSource = new DynamicPcmStreamSource(SAMPLE_RATE);
            dynamicPcmStreamSource.SampleProvider = mixer;
            mediaElement.SetSource(dynamicPcmStreamSource);

            base.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            // Save Waveformer Points
            Point[] points = new Point[waveformer.Points.Count];
            waveformer.Points.CopyTo(points, 0);
            XmlSerializer xmlSerializer = new XmlSerializer(typeof(Point[]));

            using (IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (IsolatedStorageFileStream stream = storage.CreateFile("settings.xml"))
                {
                    xmlSerializer.Serialize(stream, points);
                }
            }

            base.OnNavigatedFrom(args);
        }

        void OnMediaElementCurrentStateChanged(object sender, RoutedEventArgs args)
        {
            playButton.IsEnabled = mediaElement.CurrentState != MediaElementState.Playing;
            pauseButton.IsEnabled = mediaElement.CurrentState == MediaElementState.Playing;
        }

        void OnPlayButtonClick(object sender, EventArgs args)
        {
            mediaElement.Play();
            playButton.IsEnabled = false;
        }

        void OnPauseButtonClick(object sender, EventArgs args)
        {
            mediaElement.Pause();
            pauseButton.IsEnabled = false;
        }
    }
}